@@ -15,6 +15,7 @@
 #
 # Copyright 2014-2017  Jon Lund Steffensen <jon_steffensen@uri.edu>
 # Copyright 2015-2020  Keith Dufault-Thompson <keitht547@my.uri.edu>
+# Copyright 2020-2020  Elysha Sameth <esameth1@my.uri.edu>
 
 """Module for reading and writing native formats.
 
@@ -46,7 +47,7 @@ from .context import FilePathContext, FileMark
 from .entry import (DictCompoundEntry as CompoundEntry,
                     DictReactionEntry as ReactionEntry,
                     DictCompartmentEntry as CompartmentEntry)
-from .reaction import ReactionParser
+from .reaction import ReactionParser, convert_to_unicode
 from . import modelseed
 from .. import util
 
@@ -186,7 +187,7 @@ class ModelReader(object):
         """
         context = FilePathContext(path)
         try:
-            with open(context.filepath, 'r') as f:
+            with context.open('r') as f:
                 return ModelReader(f, context)
         except IOError:
             # Try to open the default file
@@ -194,7 +195,7 @@ class ModelReader(object):
                 try:
                     context = FilePathContext(
                         os.path.join(path, filename))
-                    with open(context.filepath, 'r') as f:
+                    with context.open('r') as f:
                         return ModelReader(f, context)
                 except:
                     logger.debug('Failed to load model file',
@@ -256,12 +257,11 @@ class ModelReader(object):
         if 'compartments' in self._model:
             boundary_map = {}
             for compartment_def in self._model['compartments']:
-                compartment_id = compartment_def.get('id')
+                compartment_id = convert_to_unicode(compartment_def.get('id'))
                 _check_id(compartment_id, 'Compartment')
                 if compartment_id in compartments:
                     raise ParseError('Duplicate compartment ID: {}'.format(
                         compartment_id))
-
                 props = dict(compartment_def)
                 adjacent_to = props.pop('adjacent_to', None)
                 if adjacent_to is not None:
@@ -370,7 +370,6 @@ class ModelReader(object):
 
     def create_model(self):
         """Return :class:`NativeModel` fully loaded into memory."""
-
         properties = {
             'name': self.name,
             'biomass': self.biomass_reaction,
@@ -432,6 +431,29 @@ class ModelReader(object):
 
         for limit in self.parse_limits():
             model.limits[limit[0]] = limit
+            dir = model.reactions.get(limit[0]).properties[
+                'equation'].direction
+            if str(dir) == 'Direction.Forward' or \
+                    str(dir) == 'Direction.Right':
+                v_min = 0
+                v_max = model.default_flux_limit
+            elif str(dir) == 'Direction.Both' or str(dir) == 'Direction.Bidir':
+                v_min = -model.default_flux_limit
+                v_max = model.default_flux_limit
+            else:
+                logger.warning('Reaction {} has invalid direction: {}'.format(
+                    limit[0], dir))
+            if (limit[1] is not None and limit[1] < v_min) or \
+                    (limit[2] is not None and limit[2] > v_max):
+                specify_vmin = limit[1] if limit[1] is not None else \
+                    -model.default_flux_limit
+                specify_vmax = limit[2] if limit[2] is not None else \
+                    -model.default_flux_limit
+                logger.warning(
+                    "Flux limits for reaction {} override default flux "
+                    "limits set by reaction direction (current flux limits: "
+                    "[{}, {}]; default limits: [{}, {}]).".format(
+                        limit[0], specify_vmin, specify_vmax, v_min, v_max))
 
         if self.has_model_definition():
             for model_reaction in self.parse_model():
@@ -610,6 +632,12 @@ class NativeModel(object):
         if len(self.model) > 0:
             model_definition = self.model
 
+        for reaction in self.reactions:
+            if reaction.equation is None:
+                logger.warning(
+                    'Reaction {} has no reaction equation'.format(reaction.id))
+                del model_definition[reaction.id]
+
         return MetabolicModel.load_model(
             database, model_definition, itervalues(self.exchange),
             itervalues(self.limits), v_max=self.default_flux_limit)
@@ -649,12 +677,18 @@ def parse_compound(compound_def, context=None):
     """Parse a structured compound definition as obtained from a YAML file
 
     Returns a CompoundEntry."""
-
-    compound_id = compound_def.get('id')
+    compound_id = convert_to_unicode(compound_def.get('id'))
     _check_id(compound_id, 'Compound')
 
+    compound_props = {}
+    for key, value in compound_def.items():
+        if isinstance(value, str):
+            compound_props[key] = convert_to_unicode(value)
+        else:
+            compound_props[key] = value
+
     mark = FileMark(context, None, None)
-    return CompoundEntry(compound_def, mark)
+    return CompoundEntry(compound_props, mark)
 
 
 def parse_compound_list(path, compounds):
@@ -662,17 +696,17 @@ def parse_compound_list(path, compounds):
 
     Yields CompoundEntries. Path can be given as a string or a context.
     """
-
     context = FilePathContext(path)
 
     for compound_def in compounds:
-        if 'include' in compound_def:
-            file_format = compound_def.get('format')
-            include_context = context.resolve(compound_def['include'])
+        compound_dict = dict(compound_def)
+        if 'include' in compound_dict:
+            file_format = compound_dict.get('format')
+            include_context = context.resolve(compound_dict['include'])
             for compound in parse_compound_file(include_context, file_format):
                 yield compound
         else:
-            yield parse_compound(compound_def, context)
+            yield parse_compound(compound_dict, context)
 
 
 def parse_compound_table_file(path, f):
@@ -685,11 +719,13 @@ def parse_compound_table_file(path, f):
     context = FilePathContext(path)
 
     for i, row in enumerate(csv.DictReader(f, delimiter=str('\t'))):
-        if 'id' not in row or row['id'].strip() == '':
+        if text_type('id') not in row or \
+                text_type(convert_to_unicode(row['id'].strip())) == '':
             raise ParseError('Expected `id` column in table')
 
-        props = {key: value for key, value in iteritems(row) if value != ''}
-
+        props = {key: text_type(convert_to_unicode(value))
+                 for key, value in iteritems(row)
+                 if text_type(convert_to_unicode(value)) != ''}
         if 'charge' in props:
             props['charge'] = int(props['charge'])
 
@@ -702,7 +738,6 @@ def parse_compound_yaml_file(path, f):
 
     Path can be given as a string or a context.
     """
-
     return parse_compound_list(path, yaml_load(f))
 
 
@@ -768,7 +803,6 @@ def parse_reaction_equation_string(equation, default_compartment):
                   if c.compartment is None else (c, v))
                  for c, v in reaction.right)
         return Reaction(reaction.direction, left, right)
-
     eq = _REACTION_PARSER.parse(equation).normalized()
     return _translate_compartments(eq, default_compartment)
 
@@ -778,11 +812,10 @@ def parse_reaction_equation(equation_def, default_compartment):
 
     Returns a Reaction.
     """
-
-    def parse_compound_list(l, compartment):
+    def parse_compound_list(compound_list, compartment):
         """Parse a list of reactants or metabolites"""
-        for compound_def in l:
-            compound_id = compound_def.get('id')
+        for compound_def in compound_list:
+            compound_id = convert_to_unicode(compound_def.get('id'))
             _check_id(compound_id, 'Compound')
 
             value = compound_def.get('value')
@@ -818,14 +851,23 @@ def parse_reaction(reaction_def, default_compartment, context=None):
 
     Returns a ReactionEntry.
     """
-
-    reaction_id = reaction_def.get('id')
+    reaction_id = convert_to_unicode(reaction_def.get('id'))
     _check_id(reaction_id, 'Reaction')
 
-    reaction_props = dict(reaction_def)
+    reaction_props = {}
+    for key, value in reaction_def.items():
+        if isinstance(value, str):
+            reaction_props[key] = convert_to_unicode(value)
+        else:
+            reaction_props[key] = value
+
+    if 'genes' in reaction_def.keys():
+        if isinstance(reaction_def['genes'], list):
+            reaction_props['genes'] = [convert_to_unicode(gene)
+                                       for gene in reaction_def['genes']]
 
     # Parse reaction equation
-    if 'equation' in reaction_def:
+    if 'equation' in reaction_def.keys():
         reaction_props['equation'] = parse_reaction_equation(
             reaction_def['equation'], default_compartment)
 
@@ -839,17 +881,17 @@ def parse_reaction_list(path, reactions, default_compartment=None):
     Yields tuples of reaction ID and reaction object. Path can be given as a
     string or a context.
     """
-
     context = FilePathContext(path)
 
     for reaction_def in reactions:
-        if 'include' in reaction_def:
-            include_context = context.resolve(reaction_def['include'])
+        reaction_dict = dict(reaction_def)
+        if 'include' in reaction_dict:
+            include_context = context.resolve(reaction_dict['include'])
             for reaction in parse_reaction_file(
                     include_context, default_compartment):
                 yield reaction
         else:
-            yield parse_reaction(reaction_def, default_compartment, context)
+            yield parse_reaction(reaction_dict, default_compartment, context)
 
 
 def parse_reaction_yaml_file(path, f, default_compartment):
@@ -871,10 +913,13 @@ def parse_reaction_table_file(path, f, default_compartment):
     context = FilePathContext(path)
 
     for lineno, row in enumerate(csv.DictReader(f, delimiter=str('\t'))):
-        if 'id' not in row or row['id'].strip() == '':
+        if text_type('id') not in row or \
+                text_type(convert_to_unicode(row['id'].strip())) == '':
             raise ParseError('Expected `id` column in table')
 
-        props = {key: value for key, value in iteritems(row) if value != ''}
+        props = {key: convert_to_unicode(value)
+                 for key, value in iteritems(row)
+                 if convert_to_unicode(value) != ''}
 
         if 'equation' in props:
             props['equation'] = parse_reaction_equation_string(
@@ -919,7 +964,7 @@ def get_limits(compound_def):
         lower = fixed
         upper = fixed
     elif ('fixed' in compound_def and
-            ('lower'in compound_def or 'upper' in compound_def)):
+            ('lower' in compound_def or 'upper' in compound_def)):
         raise ParseError('Cannot use fixed and a lower or upper bound')
     else:
         lower = compound_def.get('lower', None)
@@ -937,8 +982,11 @@ def parse_exchange(exchange_def, default_compartment):
 
     for compound_def in exchange_def.get('compounds', []):
         compartment = compound_def.get('compartment', default_compartment)
-        compound = Compound(compound_def['id'], compartment=compartment)
+        compound = Compound(convert_to_unicode(compound_def['id']),
+                            compartment=compartment)
         reaction = compound_def.get('reaction')
+        if reaction:
+            reaction = convert_to_unicode(reaction)
         lower, upper = get_limits(compound_def)
         yield compound, reaction, lower, upper
 
@@ -1006,7 +1054,7 @@ def parse_exchange_table_file(f):
     """
 
     for line in f:
-        line, _, comment = line.partition('#')
+        line, _, comment = convert_to_unicode(line).partition('#')
         line = line.strip()
         if line == '':
             continue
@@ -1021,7 +1069,7 @@ def parse_exchange_table_file(f):
         fields.extend(['-']*(4-len(fields)))
         compound_id, compartment, lower, upper = fields
 
-        compound = Compound(compound_id, compartment)
+        compound = Compound(convert_to_unicode(compound_id), compartment)
         lower = float(lower) if lower != '-' else None
         upper = float(upper) if upper != '-' else None
 
@@ -1079,7 +1127,7 @@ def parse_limit(limit_def):
     """
 
     lower, upper = get_limits(limit_def)
-    reaction = limit_def.get('reaction')
+    reaction = convert_to_unicode(limit_def.get('reaction'))
 
     return reaction, lower, upper
 
@@ -1111,7 +1159,7 @@ def parse_limits_table_file(f):
     """
 
     for line in f:
-        line, _, comment = line.partition('#')
+        line, _, comment = convert_to_unicode(line).partition('#')
         line = line.strip()
         if line == '':
             continue
@@ -1129,7 +1177,7 @@ def parse_limits_table_file(f):
         lower = float(lower) if lower != '-' else None
         upper = float(upper) if upper != '-' else None
 
-        yield reaction_id, lower, upper
+        yield convert_to_unicode(reaction_id), lower, upper
 
 
 def parse_limits_yaml_file(path, f):
@@ -1177,7 +1225,7 @@ def parse_model_group(path, group):
     context = FilePathContext(path)
 
     for reaction_id in group.get('reactions', []):
-        yield reaction_id
+        yield convert_to_unicode(reaction_id)
 
     # Parse subgroups
     for reaction_id in parse_model_group_list(
@@ -1217,7 +1265,7 @@ def parse_model_table_file(path, f):
     """
 
     for line in f:
-        line, _, comment = line.partition('#')
+        line, _, comment = convert_to_unicode(line).partition('#')
         line = line.strip()
         if line == '':
             continue
@@ -1496,3 +1544,34 @@ class ModelWriter(object):
         """
         self._write_entries(
             stream, reactions, self.convert_reaction_entry, properties)
+
+
+def reaction_signature(eq, direction=False, stoichiometry=False):
+    """Return unique signature object for :class:`Reaction`.
+
+    Signature objects are hashable, and compare equal only if the reactions
+    are considered the same according to the specified rules.
+
+    Args:
+        direction: Include reaction directionality when considering equality.
+        stoichiometry: Include stoichiometry when considering equality.
+    """
+    def compounds_sig(compounds):
+        if stoichiometry:
+            return tuple(sorted(compounds))
+        else:
+            return tuple(sorted(compound for compound, _ in compounds))
+
+    left = compounds_sig(eq.left)
+    right = compounds_sig(eq.right)
+
+    if left < right:
+        reaction_sig = left, right
+        direction_sig = eq.direction
+    else:
+        reaction_sig = right, left
+        direction_sig = eq.direction.flipped()
+
+    if direction:
+        return reaction_sig, direction_sig
+    return reaction_sig
