@@ -29,13 +29,13 @@ from .util import create_unique_id
 
 def create_exchange_id(existing_ids, compound):
     """Create unique ID for exchange of compound."""
-    return create_unique_id('EX_{}'.format(compound), existing_ids)
+    return create_unique_id(u'EX_{}'.format(compound), existing_ids)
 
 
 def create_transport_id(existing_ids, compound_1, compound_2):
     """Create unique ID for transport reaction of compounds."""
     return create_unique_id(
-        'TP_{}_{}'.format(compound_1, compound_2), existing_ids)
+        u'TP_{}_{}'.format(compound_1, compound_2), existing_ids)
 
 
 class FluxBounds(object):
@@ -133,7 +133,7 @@ class FluxBounds(object):
         return not self == other
 
     def __repr__(self):
-        return str('{}({!r}, {!r})').format(
+        return u'{}({!r}, {!r})'.format(
             self.__class__.__name__, self.lower, self.upper)
 
 
@@ -209,19 +209,19 @@ class MetabolicModel(MetabolicDatabase):
 
     def get_reaction(self, reaction_id):
         if reaction_id not in self._reaction_set:
-            raise ValueError('Reaction not in model: {}'.format(reaction_id))
+            raise ValueError(u'Reaction not in model: {}'.format(reaction_id))
         return self._database.get_reaction(reaction_id)
 
     def get_reaction_values(self, reaction_id):
         """Return stoichiometric values of reaction as a dictionary"""
         if reaction_id not in self._reaction_set:
-            raise ValueError('Unknown reaction: {}'.format(repr(reaction_id)))
+            raise ValueError(u'Unknown reaction: {}'.format(repr(reaction_id)))
         return self._database.get_reaction_values(reaction_id)
 
     def get_compound_reactions(self, compound_id):
         """Iterate over all reaction ids the includes the given compound"""
         if compound_id not in self._compound_set:
-            raise ValueError('Compound not in model: {}'.format(compound_id))
+            raise ValueError(u'Compound not in model: {}'.format(compound_id))
 
         for reaction_id in self._database.get_compound_reactions(compound_id):
             if reaction_id in self._reaction_set:
@@ -230,7 +230,7 @@ class MetabolicModel(MetabolicDatabase):
     def is_reversible(self, reaction_id):
         """Whether the given reaction is reversible"""
         if reaction_id not in self._reaction_set:
-            raise ValueError('Reaction not in model: {}'.format(reaction_id))
+            raise ValueError(u'Reaction not in model: {}'.format(reaction_id))
         return self._database.is_reversible(reaction_id)
 
     def is_exchange(self, reaction_id):
@@ -328,6 +328,159 @@ class MetabolicModel(MetabolicDatabase):
 
         return model
 
+    def make_irreversible(self, gene_dict={}, exclude_list=[], lumped_rxns={},
+                          all_reversible=False):
+        """Creates a new metabolic models with only irreversible reactions.
+
+        This function will find every reversible reaction in the
+        model and split it into two reactions with the
+        {rxnid}_forward or {rxnid}_reverse as the IDs.
+
+        Args:
+            self: A metabolicmodel object
+            gene_dict: A dictionary stores reaction ids as keys and
+                       corresponding gene association as values, it's
+                       required only in GIMME function
+            exclude_list: list of reactions to exclude in TMFA simulation
+            all_reversible: if True make all reactions in model reversible.
+
+        Values:
+            mm_irrev: A new metabolic model with only irreversible reactions
+            gene_dict_reversible: A dictionary mapping irreversible reaction
+                                  ids to gene associations, used only
+                                  in GIMME function
+            split_rxns: A list of splitted reactions, each element is a tuple
+                        of ({rxnid}_forward, {rxnid}_reverse)
+        """
+        split_reversible = set()
+        mm_irrev = self.copy()
+        reversible_gene_dict = {}
+        new_lump_rxn_dict = {}
+        for rxn in self.reactions:
+            upper = self.limits[rxn].upper
+            lower = self.limits[rxn].lower
+            mm_irrev.limits[rxn].upper = upper
+            mm_irrev.limits[rxn].lower = lower
+
+            reaction = mm_irrev.get_reaction(rxn)
+            if rxn not in exclude_list:
+                r = Reaction(Direction.Forward, reaction.left, reaction.right)
+                r2 = Reaction(Direction.Forward, reaction.right, reaction.left)
+                r_id = u'{}_forward'.format(rxn)
+                r2_id = u'{}_reverse'.format(rxn)
+                if reaction.direction == Direction.Forward:
+                    if all_reversible is False:
+                        reversible_gene_dict[rxn] = gene_dict.get(rxn)
+                        continue
+                    else:
+                        mm_irrev.remove_reaction(rxn)
+                        mm_irrev.database.set_reaction(r_id, r)
+                        mm_irrev.database.set_reaction(r2_id, r2)
+                        mm_irrev.add_reaction(r_id)
+                        mm_irrev.add_reaction(r2_id)
+                        split_reversible.add((r_id, r2_id))
+                        reversible_gene_dict[r_id] = gene_dict.get(rxn)
+                        reversible_gene_dict[r2_id] = gene_dict.get(rxn)
+                elif reaction.direction == Direction.Both:
+                    mm_irrev.remove_reaction(rxn)
+                    mm_irrev.database.set_reaction(r_id, r)
+                    mm_irrev.database.set_reaction(r2_id, r2)
+                    mm_irrev.add_reaction(r_id)
+                    mm_irrev.add_reaction(r2_id)
+                    split_reversible.add((r_id, r2_id))
+                    reversible_gene_dict[r_id] = gene_dict.get(rxn)
+                    reversible_gene_dict[r2_id] = gene_dict.get(rxn)
+                if lower >= 0:
+                    mm_irrev.limits[r_id].upper = upper
+                    mm_irrev.limits[r_id].lower = lower
+                    mm_irrev.limits[r2_id].upper = 0
+                    mm_irrev.limits[r2_id].lower = 0
+                elif upper <= 0:
+                    mm_irrev.limits[r_id].upper = 0
+                    mm_irrev.limits[r_id].lower = 0
+                    mm_irrev.limits[r2_id].upper = - lower
+                    mm_irrev.limits[r2_id].lower = - upper
+                else:
+                    mm_irrev.limits[r_id].upper = upper
+                    mm_irrev.limits[r_id].lower = 0
+                    mm_irrev.limits[r2_id].upper = - lower
+                    mm_irrev.limits[r2_id].lower = 0
+
+            elif rxn in lumped_rxns.keys():
+                final_sub_rxn_list = []
+                sub = lumped_rxns[rxn]
+                check = 0
+                for (x, y) in sub:
+                    rn = mm_irrev.get_reaction(x)
+                    if rn.direction != Direction.Both:
+                        check += 1
+                if reaction.direction == Direction.Forward or check != 0:
+                    mm_irrev.limits[rxn].upper = 0
+                    mm_irrev.limits[rxn].lower = 0
+                    sub_rxn_list = lumped_rxns[rxn]
+                    for entry in sub_rxn_list:
+                        (subrxn, dir) = entry
+                        final_sub_rxn_list.append(subrxn)
+                    new_lump_rxn_dict[rxn] = final_sub_rxn_list
+                elif reaction.direction == Direction.Both:
+                    # split the lump reaction itself
+                    r = Reaction(Direction.Forward, reaction.left,
+                                 reaction.right)
+                    r2 = Reaction(Direction.Forward, reaction.right,
+                                  reaction.left)
+                    r_id = u'{}_forward'.format(rxn)
+                    r2_id = u'{}_reverse'.format(rxn)
+                    mm_irrev.remove_reaction(rxn)
+                    mm_irrev.database.set_reaction(r_id, r)
+                    mm_irrev.database.set_reaction(r2_id, r2)
+                    mm_irrev.add_reaction(r_id)
+                    mm_irrev.add_reaction(r2_id)
+                    split_reversible.add((r_id, r2_id))
+
+                    sub_rxn_list = lumped_rxns[rxn]
+                    for_sub_rxn_list = []
+                    rev_sub_rxn_list = []
+                    mm_irrev.limits[r_id].upper = 0
+                    mm_irrev.limits[r_id].lower = 0
+                    mm_irrev.limits[r2_id].upper = 0
+                    mm_irrev.limits[r2_id].lower = 0
+                    for entry in sub_rxn_list:
+                        (subrxn, dir) = entry
+                        dir = int(dir)
+                        # subreaction = mm.get_reaction(subrxn)
+                        subreaction = self.get_reaction(subrxn)
+                        sub_r1 = Reaction(Direction.Forward,
+                                          subreaction.left, subreaction.right)
+                        sub_r1_id = u'{}_forward'.format(subrxn)
+                        sub_r2 = Reaction(Direction.Forward,
+                                          subreaction.right, subreaction.left)
+                        sub_r2_id = u'{}_reverse'.format(subrxn)
+
+                        split_reversible.add((sub_r1_id, sub_r2_id))
+                        if dir == 1:
+                            for_sub_rxn_list.append(sub_r1_id)
+                            rev_sub_rxn_list.append(sub_r2_id)
+                            mm_irrev.database.set_reaction(sub_r1_id, sub_r1)
+                            mm_irrev.add_reaction(sub_r1_id)
+                            mm_irrev.database.set_reaction(sub_r2_id, sub_r2)
+                            mm_irrev.add_reaction(sub_r2_id)
+                        elif dir == -1:
+                            for_sub_rxn_list.append(sub_r2_id)
+                            rev_sub_rxn_list.append(sub_r1_id)
+                            mm_irrev.database.set_reaction(sub_r1_id, sub_r1)
+                            mm_irrev.add_reaction(sub_r1_id)
+                            mm_irrev.database.set_reaction(sub_r2_id, sub_r2)
+                            mm_irrev.add_reaction(sub_r2_id)
+                        mm_irrev.limits[sub_r1_id].lower = 0
+                        mm_irrev.limits[sub_r1_id].upper = 100
+                        mm_irrev.limits[sub_r2_id].lower = 0
+                        mm_irrev.limits[sub_r2_id].upper = 100
+                    new_lump_rxn_dict[r_id] = for_sub_rxn_list
+                    new_lump_rxn_dict[r2_id] = rev_sub_rxn_list
+
+        return \
+            mm_irrev, reversible_gene_dict, split_reversible, new_lump_rxn_dict
+
 
 class FlipableFluxBounds(FluxBounds):
     """FluxBounds object for a FlipableModelView.
