@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of PSAMM.
 #
 # PSAMM is free software: you can redistribute it and/or modify
@@ -15,6 +16,7 @@
 #
 # Copyright 2015  Jon Lund Steffensen <jon_steffensen@uri.edu>
 # Copyright 2015-2020  Keith Dufault-Thompson <keitht547@my.uri.edu>
+# Copyright 2020-2020  Elysha Sameth <esameth@my.uri.edu>
 
 from __future__ import unicode_literals
 
@@ -26,7 +28,7 @@ import tempfile
 from contextlib import contextmanager
 import unittest
 
-from six import StringIO, BytesIO
+from six import StringIO, BytesIO, text_type
 
 from psamm.command import (main, Command, MetabolicMixin, SolverCommandMixin,
                            CommandError)
@@ -52,7 +54,7 @@ from psamm.commands.robustness import RobustnessCommand
 from psamm.commands.sbmlexport import SBMLExport
 from psamm.commands.search import SearchCommand
 from psamm.commands.tableexport import ExportTableCommand
-
+from psamm.commands.vis import VisualizationCommand
 
 @contextmanager
 def redirected_stdout(target=None):
@@ -361,11 +363,11 @@ class TestCommandMain(unittest.TestCase, BaseCommandTest):
         self.run_solver_command(FluxConsistencyCommand, ['--reduce-lp'])
 
     def test_run_fluxcheck_with_reduce_lp_infeasible(self):
-            self.skip_test_if_no_solver()
-            with self.assertRaises(SystemExit):
-                self.run_solver_command(
-                    FluxConsistencyCommand, ['--reduce-lp'],
-                    model=self._infeasible_model)
+        self.skip_test_if_no_solver()
+        with self.assertRaises(SystemExit):
+            self.run_solver_command(
+                FluxConsistencyCommand, ['--reduce-lp'],
+                model=self._infeasible_model)
 
     def test_run_fluxcheck_with_both_tfba_and_fastcore(self):
         self.skip_test_if_no_solver()
@@ -535,15 +537,48 @@ class TestCommandMain(unittest.TestCase, BaseCommandTest):
         self.run_solver_command(
             RobustnessCommand, ['--loop-removal=l1min', 'rxn_2_\u03c0'])
 
-    def test_run_robustness_with_infeasible(self):
+    def test_run_robustness_with_fva(self):
+        self.run_solver_command(
+            RobustnessCommand, ['--fva', 'rxn_2_\u03c0']
+        )
+
+    def test_run_robustness_with_invalid_objective(self):
+        with self.assertRaises(SystemExit):
+            self.run_solver_command(
+                RobustnessCommand, ['--objective=rxn_4', 'rxn_2_\u03c0'],
+                model=self._model)
+
+    def test_run_robustness_with_neg_steps(self):
+        with self.assertRaises(CommandError):
+            self.run_solver_command(
+                RobustnessCommand, ['--steps=0', 'rxn_2_\u03c0'])
+
+    def test_run_robustness_with_loop_removal_fva(self):
         self.skip_test_if_no_solver()
+        with self.assertRaises(SystemExit):
+            self.run_solver_command(
+                RobustnessCommand, ['--loop-removal=tfba', '--fva', 'rxn_2_\u03c0'])
+
+    def test_run_robustness_min_greater(self):
+        self.skip_test_if_no_solver()
+        with self.assertRaises(CommandError):
+            self.run_solver_command(
+                RobustnessCommand, ['--minimum=50', '--maximum=20', 'rxn_2_\u03c0'])
+
+    def test_run_robustness_with_infeasible(self):
         with self.assertRaises(SystemExit):
             self.run_solver_command(
                 RobustnessCommand, ['rxn_2_\u03c0'],
                 model=self._infeasible_model)
 
     def test_run_sbmlexport(self):
-        self.run_command(SBMLExport)
+        dest = tempfile.mkdtemp()
+        try:
+            dest_path = os.path.join(dest, 'model.xml')
+            self.run_command(SBMLExport, [dest_path])
+            self.assertTrue(os.path.isfile(dest_path))
+        finally:
+            shutil.rmtree(dest)
 
     def test_run_search_compound(self):
         self.run_command(SearchCommand, ['compound', '--id', 'A_\u2206'])
@@ -571,7 +606,6 @@ class TestCommandMain(unittest.TestCase, BaseCommandTest):
 
     def test_run_tableexport_compounds(self):
         f = self.run_command(ExportTableCommand, ['compounds'])
-
         self.assertTableOutputEqual(f.getvalue(), [
             ['id', 'name', 'charge', 'formula', 'in_model'],
             ['A_\u2206', 'Compound A', '0', '', 'true'],
@@ -607,6 +641,143 @@ class TestCommandMain(unittest.TestCase, BaseCommandTest):
             ['Default Flux Limits', '1000']
         ])
 
+    def test_run_vis(self):
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_element_all(self):
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                "--element", "all"])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_hide_edges(self):
+        path = os.path.join(tempfile.mkdtemp(), 'hide_edges.csv')
+        with open(path, 'w') as f:
+            f.write('{}\t{}'.format('D[c]', 'E[c]'))
+
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                "--hide-edges", path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_recolor(self):
+        path = os.path.join(tempfile.mkdtemp(), 'color.csv')
+        with open(path, 'w') as f:
+            f.write('{}\t{}'.format('B', '#f4fc55'))
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                "--color", path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_output(self):
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'test')
+        dest_path_dot = os.path.join(dest, 'test.dot')
+        dest_path_nodes = os.path.join(dest, 'test.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'test.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_compartment(self):
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                '--compartment'])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_fba(self):
+        path = os.path.join(tempfile.mkdtemp(), 'fba.tsv')
+        with open(path, 'w') as f:
+            f.write('{}\t{}'.format('rxn_1', -0.000001))
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                '--fba', path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_fba_invalid_flux(self):
+        path = os.path.join(tempfile.mkdtemp(), 'fba.tsv')
+        with open(path, 'w') as f:
+            f.write('{}\t{}'.format('rxn_1', 'a'))
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                '--fba', path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_fva(self):
+        path = os.path.join(tempfile.mkdtemp(), 'fva.tsv')
+        with open(path, 'w') as f:
+            f.write('{}\t{}\t{}'.format('rxn_1', -0.000001, 0.000001))
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                '--fva', path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
+    def test_run_vis_fva_invalid_flux(self):
+        path = os.path.join(tempfile.mkdtemp(), 'fva.tsv')
+        with open(path, 'w') as f:
+            f.write('{}\t{}\t{}'.format('rxn_1', 'a', -0.000001))
+        dest = tempfile.mkdtemp()
+        dest_path = os.path.join(dest, 'reactions')
+        dest_path_dot = os.path.join(dest, 'reactions.dot')
+        dest_path_nodes = os.path.join(dest, 'reactions.nodes.tsv')
+        dest_path_edges = os.path.join(dest, 'reactions.edges.tsv')
+        self.run_command(VisualizationCommand, ['--output', dest_path,
+                                                '--fva', path])
+        self.assertTrue(os.path.isfile(dest_path_dot))
+        self.assertTrue(os.path.isfile(dest_path_nodes))
+        self.assertTrue(os.path.isfile(dest_path_edges))
+
     def test_command_main(self):
         self.run_command(MockCommand)
         self.assertTrue(MockCommand.run_called)
@@ -700,4 +871,4 @@ class TestSBMLCommandMain(unittest.TestCase, BaseCommandTest):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    unittest.main()
\ No newline at end of file
